@@ -1,4 +1,5 @@
-# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2012-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,20 +15,19 @@
 #
 # Authors:
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2017
-# - Martin Barisits <martin.barisits@cern.ch>, 2012-2018
+# - Martin Barisits <martin.barisits@cern.ch>, 2012-2021
 # - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2017
-# - Vincent Garonne <vgaronne@gmail.com>, 2012-2018
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2018
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2013
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2020
-# - Wen Guan <wguan.icedew@gmail.com>, 2014-2015
-# - Tobias Wegner <twegner@cern.ch>, 2018
+# - Wen Guan <wen.guan@cern.ch>, 2014-2015
+# - Tobias Wegner <twegner@cern.ch>, 2018-2019
+# - Frank Berghaus <berghaus@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Eric Vaandering <ewv@fnal.gov>, 2020
-#
-# PY3K COMPATIBLE
 
 """
     Exceptions used with Rucio.
@@ -577,16 +577,6 @@ class RSEWriteBlocked(RucioException):
         self.error_code = 50
 
 
-class RSEBlacklisted(RucioException):
-    """
-    RucioException: To be deprecated at a later time, replaced by RSEWriteBlocked
-    """
-    def __init__(self, *args, **kwargs):
-        super(RSEBlacklisted, self).__init__(*args, **kwargs)
-        self._message = "RSE excluded; not available for writing."
-        self.error_code = 50
-
-
 class RSENotConnected(RucioException):
     """
     RucioException
@@ -1023,7 +1013,7 @@ class VONotFound(RucioException):
     """
     def __init__(self, *args, **kwargs):
         super(VONotFound, self).__init__(*args, **kwargs)
-        self.message = 'The requested VO does not exist'
+        self._message = 'The requested VO does not exist'
         self.error_code = 96
 
 
@@ -1033,5 +1023,35 @@ class UnsupportedAccountName(RucioException):
     """
     def __init__(self, *args, **kwargs):
         super(UnsupportedAccountName, self).__init__(*args, **kwargs)
-        self.message = 'The requested account name cannot be used'
+        self._message = 'The requested account name cannot be used'
         self.error_code = 97
+
+
+class DuplicateCriteriaInDIDFilter(RucioException):
+    """
+    Duplicate criteria found in DID filter.
+    """
+    def __init__(self, *args, **kwargs):
+        super(DuplicateCriteriaInDIDFilter, self).__init__(*args, **kwargs)
+        self._message = 'Duplicate criteria for key/operator in filter expression: {}'.format(args[0])
+        self.error_code = 98
+
+
+class DIDFilterSyntaxError(RucioException):
+    """
+    DID filter is not parsable.
+    """
+    def __init__(self, *args, **kwargs):
+        super(DIDFilterSyntaxError, self).__init__(*args, **kwargs)
+        self._message = 'Syntax error in filter expression.'
+        self.error_code = 99
+
+
+class InvalidAlgorithmName(RucioException):
+    """
+    The given algorithm name is not valid for the VO.
+    """
+    def __init__(self, algorithm, vo, *args, **kwargs):
+        super(InvalidAlgorithmName, self).__init__(*args, **kwargs)
+        self.message = 'Algorithm name %s is not valid for VO %s' % (algorithm, vo)
+        self.error_code = 100
