@@ -1,4 +1,5 @@
-# Copyright 2017-2019 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2017-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,23 +14,23 @@
 # limitations under the License.
 #
 # Authors:
-# - Vincent Garonne <vgaronne@gmail.com>, 2017
-# - Eric Vaandering <ericvaandering@gmail.com>, 2018
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2017
+# - Eric Vaandering <ewv@fnal.gov>, 2018-2021
 # - sartiran <root@polgrid107.in2p3.fr>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
 # - Sartirana Andrea <sartiran@llr.in2p3.fr>, 2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2019
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2019-2021
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2019-2021
+# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2020
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-#
-# PY3K COMPATIBLE
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 from jsonschema import validate, ValidationError
 
 from rucio.common.exception import InvalidObject
 
-
 ACCOUNT_LENGTH = 25
 
 ACCOUNT = {"description": "Account name",
@@ -54,13 +55,13 @@ ACTIVITY = {"description": "Activity name",
                      "Production Input", "Production Output",
                      "Analysis Input", "Analysis Output", "Staging",
                      "T0 Export", "T0 Tape", "Upload/Download (Job)",
-                     "Upload/Download (User)", "User Subscriptions"]}
+                     "Upload/Download (User)", "User Subscriptions", "Data Challenge"]}
 
 SCOPE_LENGTH = 25
 
 SCOPE = {"description": "Scope name",
          "type": "string",
-         "pattern": r"^(cms)|(user\.[a-z0-9-_]{1,%s})$" % (SCOPE_LENGTH - len('user.'))}
+         "pattern": r"^(cms)|(logs)|(user\.[a-z0-9-_]{1,%s})$" % (SCOPE_LENGTH - len('user.'))}
 
 R_SCOPE = {"description": "Scope name",
            "type": "string",
@@ -94,6 +95,9 @@ ASK_APPROVAL = {"description": "Rule approval request",
 ASYNCHRONOUS = {"description": "Asynchronous rule creation",
                 "type": ["boolean", "null"]}
 
+DELAY_INJECTION = {"description": "Time (in seconds) to wait before starting applying the rule. Implies asynchronous rule creation.",
+                   "type": ["integer", "null"]}
+
 PURGE_REPLICAS = {"description": "Rule purge replica status",
                   "type": "boolean"}
 
@@ -202,6 +206,7 @@ RULE = {"description": "Replication rule",
                        "comment": COMMENT,
                        "ask_approval": ASK_APPROVAL,
                        "asynchronous": ASYNCHRONOUS,
+                       "delay_injection": DELAY_INJECTION,
                        "priority": PRIORITY,
                        'split_container': SPLIT_CONTAINER,
                        'meta': METADATA},
@@ -265,13 +270,6 @@ DID = {"description": "Data Identifier(DID)",
             "then": {"properties": {"name": {"pattern": CMS_LFN}}}},
            {"if": {"properties": {"type": {"const": "F"}}},
             "then": {"properties": {"name": {"pattern": CMS_LFN}}}},
-           {"if": {"allOf": [
-               {"properties": {"scope": {"pattern": "^user\\."}}},
-               {"properties": {"type": {"const": "FILE"}}},
-           ], },
-               "then": {"properties": {"name": {"pattern": "^/store/user/rucio/"}}}},
-           {"if": {"properties": {"scope": {"const": "cms"}}},
-            "then": {"properties": {"name": {"not": {"pattern": "^/store/user/"}}}}},
        ],
        "required": ["scope", "name", "type"],
        "additionalProperties": False}
@@ -301,13 +299,13 @@ DIDS = {"description": "Array of Data Identifiers(DIDs)",
         "type": "array",
         "items": DID,
         "minItems": 1,
-        "maxItems": 1000}
+        "maxItems": 3000}  # Was 1000
 
 R_DIDS = {"description": "Array of Data Identifiers(DIDs)",
           "type": "array",
           "items": R_DID,
           "minItems": 1,
-          "maxItems": 1000}
+          "maxItems": 30000}
 
 ATTACHMENT = {"description": "Attachement",
               "type": "object",
@@ -324,7 +322,7 @@ ATTACHMENTS = {"description": "Array of attachments",
                "type": "array",
                "items": ATTACHMENT,
                "minItems": 1,
-               "maxItems": 1000}
+               "maxItems": 3000}  # Was 1000
 
 SUBSCRIPTION_FILTER = {"type": "object",
                        "properties": {"datatype": {"type": "array"},
@@ -474,3 +472,10 @@ def validate_cms_did(obj):
         _, user = scope.split('.', 1)
         if not lfn.startswith('/store/user/rucio/%s/' % user):
             raise InvalidObject("Problem with LFN %(lfn)s : Not allowed for user %(user)s" % locals())
+    if lfn.startswith('/store/user') and not lfn.startswith('/store/user/rucio/'):
+        raise InvalidObject("Problem with LFN %(lfn)s : Legacy user files are not managed with Rucio")
+    if lfn.startswith('/store/user/rucio') and not scope.startswith('user.'):
+        raise InvalidObject("Problem with LFN %(lfn)s : Only user scopes allowed in /store/user/rucio")
+    if scope == 'logs' and did_type == 'FILE':
+        if not lfn.startswith('/store/logs/'):
+            raise InvalidObject("Problem with LFN %(lfn)s : Logs must start with /store/logs" % locals())
