@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020-2021 CERN
+# Copyright 2020-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,22 +14,30 @@
 # limitations under the License.
 #
 # Authors:
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2020
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2020-2022
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Martin Barisits <martin.barisits@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
+# - martynia <martynia@users.noreply.github.com>, 2022
 
 from __future__ import print_function
+import re
+
+from json import loads
+from json.decoder import JSONDecodeError
+
 from sqlalchemy.orm.exc import NoResultFound
 from rucio.db.sqla import models
 from rucio.db.sqla.session import transactional_session, read_session
 from rucio.db.sqla.constants import DIDType
-from rucio.common.exception import InvalidType, UnsupportedOperation
+from rucio.common.exception import InvalidType, UnsupportedOperation, ConfigNotFound, RucioException
 from rucio.common.types import InternalScope, InternalAccount
 from rucio.common.utils import extract_scope
+from rucio.core.config import get as config_get
 from rucio.core.did import add_did, attach_dids_to_dids
 from rucio.core.replica import add_replicas
-from rucio.core.rule import add_rule
+from rucio.core.rule import add_rule, list_rules, update_rule
 from rucio.core.scope import list_scopes
 
 
@@ -51,7 +59,7 @@ def _exists(scope, name, session=None):
 
 
 @transactional_session
-def add_files(lfns, account, ignore_availability, session=None):
+def add_files(lfns, account, ignore_availability, vo='def', session=None):
     """
     Bulk add files :
     - Create the file and replica.
@@ -61,18 +69,31 @@ def add_files(lfns, account, ignore_availability, session=None):
     :param lfns: List of lfn (dictionary {'lfn': <lfn>, 'rse': <rse>, 'bytes': <bytes>, 'adler32': <adler32>, 'guid': <guid>, 'pfn': <pfn>}
     :param issuer: The issuer account.
     :param ignore_availability: A boolean to ignore blocklisted sites.
-    :session: The session used
+    :param vo: The VO to act on
+    :param session: The session used
     """
+    rule_extension_list = []
     attachments = []
     # The list of scopes is necessary for the extract_scope
-    scopes = list_scopes(session=session)
+    filter_ = {'scope': InternalScope(scope='*', vo=vo)}
+    scopes = list_scopes(filter_=filter_, session=session)
     scopes = [scope.external for scope in scopes]
     exist_lfn = []
+    try:
+        lifetime_dict = config_get(section='dirac', option='lifetime', session=session)
+        lifetime_dict = loads(lifetime_dict)
+    except ConfigNotFound:
+        lifetime_dict = {}
+    except JSONDecodeError as err:
+        raise InvalidType('Problem parsing lifetime option in dirac section : %s' % str(err))
+    except Exception as err:
+        raise RucioException(str(err))
+
     for lfn in lfns:
         # First check if the file exists
         filename = lfn['lfn']
         lfn_scope, _ = extract_scope(filename, scopes)
-        lfn_scope = InternalScope(lfn_scope)
+        lfn_scope = InternalScope(lfn_scope, vo=vo)
 
         exists, did_type = _exists(lfn_scope, filename)
         if exists:
@@ -87,19 +108,31 @@ def add_files(lfns, account, ignore_availability, session=None):
         # The parent must be a dataset. Register it as well as the rule
         dsn_name = lpns[0]
         dsn_scope, _ = extract_scope(dsn_name, scopes)
-        dsn_scope = InternalScope(dsn_scope)
+        dsn_scope = InternalScope(dsn_scope, vo=vo)
+
+        # Compute lifetime
+        lifetime = None
+        if dsn_scope in lifetime_dict:
+            lifetime = lifetime_dict[dsn_scope]
+        else:
+            for pattern in lifetime_dict:
+                if re.match(pattern, dsn_scope):
+                    lifetime = lifetime_dict[pattern]
+                    break
+
         exists, did_type = _exists(dsn_scope, dsn_name)
         if exists and did_type == DIDType.CONTAINER:
             raise UnsupportedOperation('Cannot create %s as dataset' % dsn_name)
         if (dsn_name not in exist_lfn) and not exists:
             print('Will create %s' % dsn_name)
-            add_did(scope=dsn_scope,
-                    name=dsn_name,
-                    did_type=DIDType.DATASET,
-                    account=InternalAccount(account),
+            # to maintain a compatibility between master and LTS-1.26 branches remove keywords for first 3 arguments
+            add_did(dsn_scope,
+                    dsn_name,
+                    DIDType.DATASET,
+                    account=InternalAccount(account, vo=vo),
                     statuses=None,
                     meta=None,
-                    rules=[{'copies': 1, 'rse_expression': 'ANY=true', 'weight': None, 'account': InternalAccount(account), 'lifetime': None, 'grouping': 'NONE'}],
+                    rules=[{'copies': 1, 'rse_expression': 'ANY=true', 'weight': None, 'account': InternalAccount(account, vo=vo), 'lifetime': None, 'grouping': 'NONE'}],
                     lifetime=None,
                     dids=None,
                     rse_id=None,
@@ -107,8 +140,15 @@ def add_files(lfns, account, ignore_availability, session=None):
             exist_lfn.append(dsn_name)
             parent_name = lpns[1]
             parent_scope, _ = extract_scope(parent_name, scopes)
-            parent_scope = InternalScope(parent_scope)
+            parent_scope = InternalScope(parent_scope, vo=vo)
             attachments.append({'scope': parent_scope, 'name': parent_name, 'dids': [{'scope': dsn_scope, 'name': dsn_name}]})
+            rule_extension_list.append((dsn_scope, dsn_name))
+        if lifetime and (dsn_scope, dsn_name) not in rule_extension_list:
+            # Reset the lifetime of the rule to the configured value
+            rule = [rul for rul in list_rules({'scope': dsn_scope, 'name': dsn_name, 'account': InternalAccount(account, vo=vo)}, session=session) if rul['rse_expression'] == 'ANY=true']
+            if rule:
+                update_rule(rule[0]['id'], options={'lifetime': lifetime}, session=session)
+            rule_extension_list.append((dsn_scope, dsn_name))
 
         # Register the file
         rse_id = lfn.get('rse_id', None)
@@ -126,11 +166,11 @@ def add_files(lfns, account, ignore_availability, session=None):
         add_replicas(rse_id=rse_id,
                      files=[files],
                      dataset_meta=None,
-                     account=InternalAccount(account),
+                     account=InternalAccount(account, vo=vo),
                      ignore_availability=ignore_availability,
                      session=session)
         add_rule(dids=[{'scope': lfn_scope, 'name': filename}],
-                 account=InternalAccount(account),
+                 account=InternalAccount(account, vo=vo),
                  copies=1,
                  rse_expression=lfn['rse'],
                  grouping=None,
@@ -144,16 +184,16 @@ def add_files(lfns, account, ignore_availability, session=None):
         # Now loop over the ascendants of the dataset and created them
         for lpn in lpns[1:]:
             child_scope, _ = extract_scope(lpn, scopes)
-            child_scope = InternalScope(child_scope)
+            child_scope = InternalScope(child_scope, vo=vo)
             exists, did_type = _exists(child_scope, lpn)
             if exists and did_type == DIDType.DATASET:
                 raise UnsupportedOperation('Cannot create %s as container' % lpn)
             if (lpn not in exist_lfn) and not exists:
                 print('Will create %s' % lpn)
-                add_did(scope=child_scope,
-                        name=lpn,
-                        did_type=DIDType.CONTAINER,
-                        account=InternalAccount(account),
+                add_did(child_scope,
+                        lpn,
+                        DIDType.CONTAINER,
+                        account=InternalAccount(account, vo=vo),
                         statuses=None,
                         meta=None,
                         rules=None,
@@ -164,10 +204,10 @@ def add_files(lfns, account, ignore_availability, session=None):
                 exist_lfn.append(lpn)
                 parent_name = lpns[lpns.index(lpn) + 1]
                 parent_scope, _ = extract_scope(parent_name, scopes)
-                parent_scope = InternalScope(parent_scope)
+                parent_scope = InternalScope(parent_scope, vo=vo)
                 attachments.append({'scope': parent_scope, 'name': parent_name, 'dids': [{'scope': child_scope, 'name': lpn}]})
     # Finally attach everything
     attach_dids_to_dids(attachments,
-                        account=InternalAccount(account),
+                        account=InternalAccount(account, vo=vo),
                         ignore_duplicate=True,
                         session=session)
