@@ -17,7 +17,7 @@
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2015
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2021
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2021
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2017
 # - Wen Guan <wen.guan@cern.ch>, 2015-2016
@@ -25,7 +25,7 @@
 # - Frank Berghaus <frank.berghaus@cern.ch>, 2018
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018-2020
+# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018-2021
 # - James Perry <j.perry@epcc.ed.ac.uk>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Brandon White <bjwhite@fnal.gov>, 2019
@@ -37,6 +37,7 @@
 # - Tomas Javurek <tomas.javurek@cern.ch>, 2020
 # - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
 # - Joel Dierkes <joel.dierkes@cern.ch>, 2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 import json
 from io import StringIO
@@ -66,7 +67,7 @@ if TYPE_CHECKING:
     from typing import Dict, Optional
     from sqlalchemy.orm import Session
 
-REGION = make_region_memcached(expiration_time=3600)
+REGION = make_region_memcached(expiration_time=900)
 
 
 @transactional_session
@@ -924,6 +925,11 @@ def add_protocol(rse_id, parameter, session=None):
                 parameter[op_name] = parameter['domains'][s][op]
         del parameter['domains']
 
+    # This is needed for forward-compatibility with the 1.28 version, which relies on _read/_write columns
+    if parameter.get('third_party_copy') is not None and parameter.get('third_party_copy_read') is None and parameter.get('third_party_copy_write') is None:
+        parameter['third_party_copy_read'] = parameter['third_party_copy']
+        parameter['third_party_copy_write'] = parameter['third_party_copy']
+
     if ('extended_attributes' in parameter) and parameter['extended_attributes']:
         try:
             parameter['extended_attributes'] = json.dumps(parameter['extended_attributes'], separators=(',', ':'))
