@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2021 CERN
+# Copyright 2013-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,16 +24,16 @@
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - James Perry <j.perry@epcc.ed.ac.uk>, 2020
 # - Eric Vaandering <ewv@fnal.gov>, 2020
+# - Radu Carpa <radu.carpa@cern.ch>, 2022
 
 import abc
 import re
 
-from dogpile.cache import make_region
 from dogpile.cache.api import NoValue
 from hashlib import sha256
 from six import add_metaclass
 
-from rucio.common.config import config_get
+from rucio.common.cache import make_region_memcached
 from rucio.common.exception import InvalidRSEExpression, RSEWriteBlocked
 from rucio.core.rse import list_rses, get_rses_with_attribute, get_rse_attribute
 from rucio.db.sqla.session import transactional_session
@@ -48,19 +48,16 @@ COMPLEMENT = r'(\\%s)' % (PRIMITIVE)
 
 PATTERN = r'^%s(%s|%s|%s)*' % (PRIMITIVE, UNION, INTERSECTION, COMPLEMENT)
 
-
-REGION = make_region().configure('dogpile.cache.memcached',
-                                 expiration_time=600,
-                                 arguments={'url': config_get('cache', 'url', False, '127.0.0.1:11211'), 'distributed_lock': True})
+REGION = make_region_memcached(expiration_time=600)
 
 
 @transactional_session
-def parse_expression(expression, filter=None, session=None):
+def parse_expression(expression, filter_=None, session=None):
     """
     Parse a RSE expression and return the list of RSE dictionaries.
 
     :param expression:    RSE expression, e.g: 'CERN|BNL'.
-    :param filter:        Availability filter (dictionary) used for the RSEs. e.g.: {'availability_write': True}
+    :param filter_:        Availability filter (dictionary) used for the RSEs. e.g.: {'availability_write': True}
     :param session:       Database session in use.
     :returns:             A list of rse dictionaries.
     :raises:              InvalidRSEExpression, RSENotFound, RSEWriteBlocked
@@ -96,9 +93,9 @@ def parse_expression(expression, filter=None, session=None):
 
     # Filter for VO
     vo_result = []
-    if filter and filter.get('vo'):
-        filter = filter.copy()  # Make a copy so we can pop('vo') without affecting the object `filter` outside this function
-        vo = filter.pop('vo')
+    if filter_ and filter_.get('vo'):
+        filter_ = filter_.copy()  # Make a copy so we can pop('vo') without affecting the object `filter` outside this function
+        vo = filter_.pop('vo')
         for rse in result:
             if rse.get('vo') == vo:
                 vo_result.append(rse)
@@ -110,9 +107,9 @@ def parse_expression(expression, filter=None, session=None):
 
     # Filter
     final_result = []
-    if filter:
+    if filter_:
         for rse in vo_result:
-            if filter.get('availability_write', False):
+            if filter_.get('availability_write', False):
                 if rse.get('availability') & 2:
                     final_result.append(rse)
         if not final_result:
