@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2021 CERN
+# Copyright 2013-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,8 @@
 # - Robert Illingworth <illingwo@fnal.gov>, 2019
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Yutaro Iiyama <yutaro.iiyama@cern.ch>, 2022
+# - Joel Dierkes <joel.dierkes@cern.ch>, 2022
 
 from random import uniform, shuffle
 
@@ -29,6 +31,7 @@ from rucio.core.account import has_account_attribute, get_usage, get_all_rse_usa
 from rucio.core.account_limit import get_local_account_limit, get_global_account_limits
 from rucio.core.rse import list_rse_attributes, has_rse_attribute, get_rse_limits
 from rucio.core.rse_counter import get_counter as get_rse_counter
+from rucio.core.rse_expression_parser import parse_expression
 from rucio.db.sqla.session import read_session
 
 
@@ -247,3 +250,35 @@ class RSESelector():
             weight += rse['weight']
             if pick <= weight:
                 return (rse['rse_id'], rse['staging_area'], rse['availability_write'])
+
+
+@read_session
+def resolve_rse_expression(rse_expression, account, weight=None, copies=1, ignore_account_limit=False, size=0, preferred_rses=[], blocklist=[], prioritize_order_over_weight=False, existing_rse_size=None, session=None):
+    """
+    Resolve a potentially complex RSE expression into `copies` single-RSE expressions. Uses `parse_expression()`
+    to decompose the expression, then `RSESelector.select_rse()` to pick the target RSEs.
+    """
+
+    rses = parse_expression(rse_expression, filter_={'vo': account.vo}, session=session)
+
+    rse_to_id = dict((rse_dict['rse'], rse_dict['id']) for rse_dict in rses)
+    id_to_rse = dict((rse_dict['id'], rse_dict['rse']) for rse_dict in rses)
+
+    selector = RSESelector(account=account,
+                           rses=rses,
+                           weight=weight,
+                           copies=copies,
+                           ignore_account_limit=ignore_account_limit,
+                           session=session)
+
+    preferred_rse_ids = [rse_to_id[rse] for rse in preferred_rses if rse in rse_to_id]
+
+    preferred_unmatched = list(set(preferred_rses) - set(rse_dict['rse'] for rse_dict in rses))
+
+    selection_result = selector.select_rse(size=size,
+                                           preferred_rse_ids=preferred_rse_ids,
+                                           blocklist=blocklist,
+                                           prioritize_order_over_weight=prioritize_order_over_weight,
+                                           existing_rse_size=existing_rse_size)
+
+    return [id_to_rse[rse_id] for rse_id, _, _ in selection_result], preferred_unmatched
