@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020-2021 CERN
+# Copyright 2020-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,10 +17,10 @@
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 # - David Población Criado <david.poblacion.criado@cern.ch>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
 
+import functools
 import logging
-import os
-import socket
 import threading
 from time import time
 from typing import TYPE_CHECKING
@@ -28,17 +28,17 @@ from typing import TYPE_CHECKING
 import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.exception import RucioException
-from rucio.common.logging import formatted_logger, setup_logging
-from rucio.common.utils import daemon_sleep
-from rucio.core import heartbeat
+from rucio.common.logging import setup_logging
 from rucio.core.request import preparer_update_requests, reduce_requests, sort_requests_minimum_distance, \
     get_transfertool_filter, get_supported_transfertools, rse_lookup_filter
 from rucio.core.transfer import __list_transfer_requests_and_source_replicas
+from rucio.daemons.conveyor.common import run_conveyor_daemon
 from rucio.db.sqla.constants import RequestState
 
 if TYPE_CHECKING:
-    from typing import Optional, Tuple
+    from typing import Optional
     from sqlalchemy.orm import Session
+    from rucio.daemons.conveyor.common import HeartbeatHandler
 
 graceful_stop = threading.Event()
 
@@ -90,63 +90,54 @@ def run(once=False, threads=1, sleep_time=10, bulk=100):
 
 def preparer(once, sleep_time, bulk, partition_wait_time=10):
     # Make an initial heartbeat so that all instanced daemons have the correct worker number on the next try
-    executable = 'conveyor-preparer'
-    hostname = socket.gethostname()
-    pid = os.getpid()
-    current_thread = threading.current_thread()
-    worker_number = current_thread
-    total_workers = '?'
-    prefix = 'conveyor-preparer[%s/%s] ' % (worker_number, total_workers)
-    daemon_logger = formatted_logger(logging.log, prefix + '%s')
-    heartbeat.sanity_check(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
+    logger_prefix = executable = 'conveyor-preparer'
 
-    try:
-        if partition_wait_time is not None:
-            graceful_stop.wait(partition_wait_time)  # gathering of daemons/threads on first start
-        while not graceful_stop.is_set():
-            start_time = time()
-
-            pulse = heartbeat.live(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
-            worker_number = pulse['assign_thread']
-            total_workers = pulse['nr_threads']
-
-            prefix = 'conveyor-preparer[%s/%s] ' % (worker_number, total_workers)
-            daemon_logger = formatted_logger(logging.log, prefix + '%s')
-
-            count = 0
-            try:
-                count, updated_msg = run_once(total_workers=total_workers, worker_number=worker_number, limit=bulk, logger=daemon_logger)
-            except RucioException:
-                daemon_logger(logging.ERROR, 'errored with a RucioException, retrying later', exc_info=True)
-                updated_msg = 'errored'
-
-            if once:
-                break
-
-            end_time = time()
-            time_diff = end_time - start_time
-            daemon_logger(logging.INFO, '%s, taking %.3f seconds' % (updated_msg, time_diff))
-            if count < bulk:
-                daemon_sleep(start_time=start_time, sleep_time=sleep_time, graceful_stop=graceful_stop, logger=daemon_logger)
-
-        daemon_logger(logging.INFO, 'gracefully stopping')
-
-    finally:
-        heartbeat.die(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
-
-
-def run_once(total_workers: int = 0, worker_number: int = 0, limit: "Optional[int]" = None, logger=logging.log, session: "Optional[Session]" = None) -> "Tuple[int, str]":
-    req_sources = __list_transfer_requests_and_source_replicas(
-        total_workers=total_workers,
-        worker_number=worker_number,
-        limit=limit,
-        request_state=RequestState.PREPARING,
-        session=session
+    run_conveyor_daemon(
+        once=once,
+        graceful_stop=graceful_stop,
+        executable=executable,
+        logger_prefix=logger_prefix,
+        partition_wait_time=partition_wait_time,
+        sleep_time=sleep_time,
+        run_once_fnc=functools.partial(
+            run_once,
+            bulk=bulk
+        ),
+        activities=None,
     )
-    if not req_sources:
-        return 0, 'had nothing to do'
 
-    transfertool_filter = get_transfertool_filter(lambda rse_id: get_supported_transfertools(rse_id=rse_id, session=session))
-    requests = reduce_requests(req_sources, [rse_lookup_filter, sort_requests_minimum_distance, transfertool_filter], logger=logger)
-    count = preparer_update_requests(requests, session=session)
-    return count, f'updated {count}/{limit} requests'
+
+def run_once(bulk: int = 100, heartbeat_handler: "Optional[HeartbeatHandler]" = None, limit: "Optional[int]" = None, session: "Optional[Session]" = None, **kwargs) -> bool:
+    if heartbeat_handler:
+        worker_number, total_workers, logger = heartbeat_handler.live()
+    else:
+        # This is used in tests
+        worker_number, total_workers, logger = 0, 0, logging.log
+
+    start_time = time()
+    try:
+        req_sources = __list_transfer_requests_and_source_replicas(
+            total_workers=total_workers,
+            worker_number=worker_number,
+            limit=limit,
+            request_state=RequestState.PREPARING,
+            session=session
+        )
+        if not req_sources:
+            count = 0
+            updated_msg = 'had nothing to do'
+        else:
+            transfertool_filter = get_transfertool_filter(lambda rse_id: get_supported_transfertools(rse_id=rse_id, session=session))
+            requests = reduce_requests(req_sources, [rse_lookup_filter, sort_requests_minimum_distance, transfertool_filter], logger=logger)
+            count = preparer_update_requests(requests, session=session)
+            updated_msg = f'updated {count}/{limit} requests'
+    except RucioException:
+        logger(logging.ERROR, 'errored with a RucioException, retrying later', exc_info=True)
+        count = 0
+        updated_msg = 'errored'
+    logger(logging.INFO, '%s, taking %.3f seconds' % (updated_msg, time() - start_time))
+
+    queue_empty = False
+    if count < bulk:
+        queue_empty = True
+    return queue_empty
