@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2021 CERN
+# Copyright 2015-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,7 +23,7 @@
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
 
 """
 Conveyor stager is a daemon to manage stagein file transfers.
@@ -31,10 +31,10 @@ Conveyor stager is a daemon to manage stagein file transfers.
 
 from __future__ import division
 
+import functools
 import logging
 import threading
 import time
-from collections import defaultdict
 
 from six.moves.configparser import NoOptionError
 
@@ -44,13 +44,55 @@ from rucio.common.config import config_get, config_get_bool, config_get_int
 from rucio.common.logging import setup_logging
 from rucio.core.monitor import record_counter, record_timer
 from rucio.core import transfer as transfer_core
-from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, HeartbeatHandler
+from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, run_conveyor_daemon
 from rucio.db.sqla.constants import RequestType
 from rucio.transfertool.fts3 import FTS3Transfertool
 
 graceful_stop = threading.Event()
 
 
+def run_once(bulk, group_bulk, rse_ids, scheme, failover_scheme, transfertool_kwargs, heartbeat_handler, activity):
+    worker_number, total_workers, logger = heartbeat_handler.live()
+
+    start_time = time.time()
+    transfers = transfer_core.next_transfers_to_submit(
+        total_workers=total_workers,
+        worker_number=worker_number,
+        failover_schemes=failover_scheme,
+        limit=bulk,
+        activity=activity,
+        rses=rse_ids,
+        schemes=scheme,
+        transfertools_by_name={'fts3': FTS3Transfertool},
+        older_than=None,
+        request_type=RequestType.STAGEIN,
+        logger=logger,
+    )
+    total_transfers = len(list(hop for paths in transfers.values() for path in paths for hop in path))
+    record_timer('daemons.conveyor.stager.get_stagein_transfers.per_transfer', (time.time() - start_time) * 1000 / (total_transfers if transfers else 1))
+    record_counter('daemons.conveyor.stager.get_stagein_transfers', total_transfers)
+    record_timer('daemons.conveyor.stager.get_stagein_transfers.transfers', total_transfers)
+    logger(logging.INFO, 'Got %s stagein transfers for %s' % (total_transfers, activity))
+
+    for builder, transfer_paths in transfers.items():
+        transfertool_obj = builder.make_transfertool(logger=logger, **transfertool_kwargs.get(builder.transfertool_class, {}))
+        logger(logging.INFO, 'Starting to group transfers for %s (%s)' % (activity, transfertool_obj))
+        start_time = time.time()
+        grouped_jobs = transfertool_obj.group_into_submit_jobs(transfer_paths)
+        record_timer('daemons.conveyor.stager.bulk_group_transfer', (time.time() - start_time) * 1000 / (len(transfer_paths) or 1))
+
+        logger(logging.INFO, 'Starting to submit transfers for %s (%s)' % (activity, transfertool_obj))
+        for job in grouped_jobs:
+            worker_number, total_workers, logger = heartbeat_handler.live()
+            submit_transfer(transfertool_obj=transfertool_obj, transfers=job['transfers'], job_params=job['job_params'], submitter='transfer_submitter', logger=logger)
+
+    queue_empty = False
+    if total_transfers < group_bulk:
+        queue_empty = True
+        logger(logging.INFO, 'Only %s transfers for %s which is less than group bulk %s' % (total_transfers, activity, group_bulk))
+    return queue_empty
+
+
 def stager(once=False, rses=None, bulk=100, group_bulk=1, group_policy='rule',
            source_strategy=None, activities=None, sleep_time=600):
     """
@@ -85,84 +127,45 @@ def stager(once=False, rses=None, bulk=100, group_bulk=1, group_policy='rule',
         max_time_in_queue['default'] = 168
     logging.debug("Maximum time in queue for different activities: %s" % max_time_in_queue)
 
-    activity_next_exe_time = defaultdict(time.time)
     logger_prefix = executable = 'conveyor-stager'
     if activities:
         activities.sort()
         executable += '--activities ' + str(activities)
-    with HeartbeatHandler(executable=executable, logger_prefix=logger_prefix) as heartbeat_handler:
-        logger = heartbeat_handler.logger
-        logger(logging.INFO, 'Stager starting with bring_online %s seconds' % (bring_online))
 
-        while not graceful_stop.is_set():
+    if rses:
+        rse_ids = [rse['id'] for rse in rses]
+    else:
+        rse_ids = None
 
-            try:
-                heart_beat, logger = heartbeat_handler.live()
+    transfertool_kwargs = {
+        FTS3Transfertool: {
+            'group_policy': group_policy,
+            'group_bulk': group_bulk,
+            'source_strategy': source_strategy,
+            'max_time_in_queue': max_time_in_queue,
+            'bring_online': bring_online,
+            'default_lifetime': -1,
+        }
+    }
 
-                if activities is None:
-                    activities = [None]
-                if rses:
-                    rse_ids = [rse['id'] for rse in rses]
-                else:
-                    rse_ids = None
-
-                for activity in activities:
-                    if activity_next_exe_time[activity] > time.time():
-                        graceful_stop.wait(1)
-                        continue
-
-                    logger(logging.INFO, 'Starting to get stagein transfers for %s' % (activity))
-                    start_time = time.time()
-
-                    transfertool_kwargs = {
-                        FTS3Transfertool: {
-                            'group_policy': group_policy,
-                            'group_bulk': group_bulk,
-                            'source_strategy': source_strategy,
-                            'max_time_in_queue': max_time_in_queue,
-                            'bring_online': bring_online,
-                            'default_lifetime': -1,
-                        }
-                    }
-                    transfers = transfer_core.next_transfers_to_submit(
-                        total_workers=heart_beat['nr_threads'],
-                        worker_number=heart_beat['assign_thread'],
-                        failover_schemes=failover_scheme,
-                        limit=bulk,
-                        activity=activity,
-                        rses=rse_ids,
-                        schemes=scheme,
-                        transfertools_by_name={'fts3': FTS3Transfertool},
-                        older_than=None,
-                        request_type=RequestType.STAGEIN,
-                        logger=logger,
-                    )
-                    total_transfers = len(list(hop for paths in transfers.values() for path in paths for hop in path))
-                    record_timer('daemons.conveyor.stager.get_stagein_transfers.per_transfer', (time.time() - start_time) * 1000 / (total_transfers if transfers else 1))
-                    record_counter('daemons.conveyor.stager.get_stagein_transfers', total_transfers)
-                    record_timer('daemons.conveyor.stager.get_stagein_transfers.transfers', total_transfers)
-                    logger(logging.INFO, 'Got %s stagein transfers for %s' % (total_transfers, activity))
-
-                    for builder, transfer_paths in transfers.items():
-                        transfertool_obj = builder.make_transfertool(logger=logger, **transfertool_kwargs.get(builder.transfertool_class, {}))
-                        logger(logging.INFO, 'Starting to group transfers for %s (%s)' % (activity, transfertool_obj))
-                        start_time = time.time()
-                        grouped_jobs = transfertool_obj.group_into_submit_jobs(transfer_paths)
-                        record_timer('daemons.conveyor.stager.bulk_group_transfer', (time.time() - start_time) * 1000 / (len(transfer_paths) or 1))
-
-                        logger(logging.INFO, 'Starting to submit transfers for %s (%s)' % (activity, transfertool_obj))
-                        for job in grouped_jobs:
-                            submit_transfer(transfertool_obj=transfertool_obj, transfers=job['transfers'], job_params=job['job_params'], submitter='transfer_submitter', logger=logger)
-
-                    if total_transfers < group_bulk:
-                        logger(logging.INFO, 'Only %s transfers for %s which is less than group bulk %s, sleep %s seconds' % (total_transfers, activity, group_bulk, sleep_time))
-                        if activity_next_exe_time[activity] < time.time():
-                            activity_next_exe_time[activity] = time.time() + sleep_time
-            except Exception:
-                raise
-
-            if once:
-                break
+    run_conveyor_daemon(
+        once=once,
+        graceful_stop=graceful_stop,
+        executable=executable,
+        logger_prefix=logger_prefix,
+        partition_wait_time=None,
+        sleep_time=sleep_time,
+        run_once_fnc=functools.partial(
+            run_once,
+            bulk=bulk,
+            group_bulk=group_bulk,
+            scheme=scheme,
+            failover_scheme=failover_scheme,
+            rse_ids=rse_ids,
+            transfertool_kwargs=transfertool_kwargs,
+        ),
+        activities=activities,
+    )
 
 
 def stop(signum=None, frame=None):
