@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2021 CERN
+# Copyright 2013-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,7 +31,7 @@
 # - James Perry <j.perry@epcc.ed.ac.uk>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
 
 """
 Conveyor transfer submitter is a daemon to manage non-tape file transfers.
@@ -39,10 +39,10 @@ Conveyor transfer submitter is a daemon to manage non-tape file transfers.
 
 from __future__ import division
 
+import functools
 import logging
 import threading
 import time
-from collections import defaultdict
 
 from six.moves.configparser import NoOptionError
 
@@ -51,10 +51,9 @@ from rucio.common import exception
 from rucio.common.config import config_get, config_get_bool, config_get_int
 from rucio.common.logging import setup_logging
 from rucio.common.schema import get_schema_value
-from rucio.common.utils import PriorityQueue
 from rucio.core import transfer as transfer_core
 from rucio.core.monitor import MultiCounter, record_timer
-from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, HeartbeatHandler
+from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, run_conveyor_daemon
 from rucio.db.sqla.constants import RequestType
 from rucio.transfertool.fts3 import FTS3Transfertool
 from rucio.transfertool.globus import GlobusTransferTool
@@ -76,6 +75,56 @@ TRANSFERTOOL_CLASSES_BY_NAME = {
 }
 
 
+def run_once(bulk, group_bulk, filter_transfertool, transfertool, ignore_availability, rse_ids,
+             scheme, failover_scheme, partition_hash_var, timeout, transfertool_kwargs,
+             heartbeat_handler, activity):
+    worker_number, total_workers, logger = heartbeat_handler.live()
+
+    start_time = time.time()
+    transfers = transfer_core.next_transfers_to_submit(
+        total_workers=total_workers,
+        worker_number=worker_number,
+        partition_hash_var=partition_hash_var,
+        failover_schemes=failover_scheme,
+        limit=bulk,
+        activity=activity,
+        rses=rse_ids,
+        schemes=scheme,
+        filter_transfertool=filter_transfertool,
+        transfertools_by_name={transfertool: TRANSFERTOOL_CLASSES_BY_NAME[transfertool]},
+        older_than=None,
+        request_type=RequestType.TRANSFER,
+        ignore_availability=ignore_availability,
+        logger=logger,
+    )
+    total_transfers = len(list(hop for paths in transfers.values() for path in paths for hop in path))
+
+    record_timer('daemons.conveyor.transfer_submitter.get_transfers.per_transfer', (time.time() - start_time) * 1000 / (total_transfers or 1))
+    GET_TRANSFERS_COUNTER.inc(total_transfers)
+    record_timer('daemons.conveyor.transfer_submitter.get_transfers.transfers', total_transfers)
+    logger(logging.INFO, 'Got %s transfers for %s in %s seconds', total_transfers, activity, time.time() - start_time)
+
+    for builder, transfer_paths in transfers.items():
+        transfertool_obj = builder.make_transfertool(logger=logger, **transfertool_kwargs.get(builder.transfertool_class, {}))
+        start_time = time.time()
+        logger(logging.DEBUG, 'Starting to group transfers for %s (%s)', activity, transfertool_obj)
+        grouped_jobs = transfertool_obj.group_into_submit_jobs(transfer_paths)
+        record_timer('daemons.conveyor.transfer_submitter.bulk_group_transfer', (time.time() - start_time) * 1000 / (len(transfer_paths) or 1))
+
+        logger(logging.DEBUG, 'Starting to submit transfers for %s (%s)', activity, transfertool_obj)
+        for job in grouped_jobs:
+            worker_number, total_workers, logger = heartbeat_handler.live()
+            logger(logging.DEBUG, 'submitjob: transfers=%s, job_params=%s' % ([str(t) for t in job['transfers']], job['job_params']))
+            submit_transfer(transfertool_obj=transfertool_obj, transfers=job['transfers'], job_params=job['job_params'], submitter='transfer_submitter',
+                            timeout=timeout, logger=logger)
+
+    queue_empty = False
+    if total_transfers < group_bulk:
+        queue_empty = True
+        logger(logging.DEBUG, 'Only %s transfers for %s which is less than group bulk %s', total_transfers, activity, group_bulk)
+    return queue_empty
+
+
 def submitter(once=False, rses=None, partition_wait_time=10,
               bulk=100, group_bulk=1, group_policy='rule', source_strategy=None,
               activities=None, sleep_time=600, max_sources=4, archive_timeout_override=None,
@@ -122,112 +171,56 @@ def submitter(once=False, rses=None, partition_wait_time=10,
         max_time_in_queue['default'] = 168
     logging.debug("Maximum time in queue for different activities: %s", max_time_in_queue)
 
-    activity_next_exe_time = defaultdict(time.time)
     logger_prefix = executable = "conveyor-submitter"
     if activities:
         activities.sort()
         executable += '--activities ' + str(activities)
     if filter_transfertool:
         executable += ' --filter-transfertool ' + filter_transfertool
-
-    if activities is None:
-        activities = [None]
     if rses:
         rse_ids = [rse['id'] for rse in rses]
     else:
         rse_ids = None
 
-    with HeartbeatHandler(executable=executable, logger_prefix=logger_prefix) as heartbeat_handler:
-        logger = heartbeat_handler.logger
-        logger(logging.INFO, 'Submitter starting with timeout %s', timeout)
+    transfertool_kwargs = {
+        FTS3Transfertool: {
+            'group_policy': group_policy,
+            'group_bulk': group_bulk,
+            'source_strategy': source_strategy,
+            'max_time_in_queue': max_time_in_queue,
+            'bring_online': bring_online,
+            'default_lifetime': 172800,
+            'archive_timeout_override': archive_timeout_override,
+        },
+        GlobusTransferTool: {
+            'group_policy': transfertype,
+            'group_bulk': group_bulk,
+        },
+    }
 
-        if partition_wait_time:
-            graceful_stop.wait(partition_wait_time)
-
-        activity_next_exe_time = PriorityQueue()
-        for activity in activities:
-            activity_next_exe_time[activity] = time.time()
-
-        while not graceful_stop.is_set() and activity_next_exe_time:
-            try:
-                time_to_sleep = 0
-                if once:
-                    activity = activity_next_exe_time.pop()
-                else:
-                    activity = activity_next_exe_time.top()
-                    time_to_sleep = activity_next_exe_time[activity] - time.time()
-                    activity_next_exe_time[activity] = time.time() + 1
-                if time_to_sleep > 0:
-                    logger(logging.DEBUG, 'Switching to activity %s and sleeping %s seconds', activity, time_to_sleep)
-                    graceful_stop.wait(time_to_sleep)
-                else:
-                    logger(logging.DEBUG, 'Switching to activity %s', activity)
-
-                heart_beat, logger = heartbeat_handler.live(older_than=3600)
-
-                start_time = time.time()
-
-                transfertool_kwargs = {
-                    FTS3Transfertool: {
-                        'group_policy': group_policy,
-                        'group_bulk': group_bulk,
-                        'source_strategy': source_strategy,
-                        'max_time_in_queue': max_time_in_queue,
-                        'bring_online': bring_online,
-                        'default_lifetime': 172800,
-                        'archive_timeout_override': archive_timeout_override,
-                    },
-                    GlobusTransferTool: {
-                        'group_policy': transfertype,
-                        'group_bulk': group_bulk,
-                    },
-                }
-                transfers = transfer_core.next_transfers_to_submit(
-                    total_workers=heart_beat['nr_threads'],
-                    worker_number=heart_beat['assign_thread'],
-                    partition_hash_var=partition_hash_var,
-                    failover_schemes=failover_scheme,
-                    limit=bulk,
-                    activity=activity,
-                    rses=rse_ids,
-                    schemes=scheme,
-                    filter_transfertool=filter_transfertool,
-                    transfertools_by_name={transfertool: TRANSFERTOOL_CLASSES_BY_NAME[transfertool]},
-                    older_than=None,
-                    request_type=RequestType.TRANSFER,
-                    ignore_availability=ignore_availability,
-                    logger=logger,
-                )
-                total_transfers = len(list(hop for paths in transfers.values() for path in paths for hop in path))
-
-                record_timer('daemons.conveyor.transfer_submitter.get_transfers.per_transfer', (time.time() - start_time) * 1000 / (total_transfers or 1))
-                GET_TRANSFERS_COUNTER.inc(total_transfers)
-                record_timer('daemons.conveyor.transfer_submitter.get_transfers.transfers', total_transfers)
-                logger(logging.INFO, '%sGot %s transfers for %s in %s seconds',
-                       'Slept %s seconds, then ' % time_to_sleep if time_to_sleep > 0 else '',
-                       total_transfers, activity, time.time() - start_time)
-
-                for builder, transfer_paths in transfers.items():
-                    transfertool_obj = builder.make_transfertool(logger=logger, **transfertool_kwargs.get(builder.transfertool_class, {}))
-                    start_time = time.time()
-                    logger(logging.DEBUG, 'Starting to group transfers for %s (%s)', activity, transfertool_obj)
-                    grouped_jobs = transfertool_obj.group_into_submit_jobs(transfer_paths)
-                    record_timer('daemons.conveyor.transfer_submitter.bulk_group_transfer', (time.time() - start_time) * 1000 / (len(transfer_paths) or 1))
-
-                    logger(logging.DEBUG, 'Starting to submit transfers for %s (%s)', activity, transfertool_obj)
-                    for job in grouped_jobs:
-                        logger(logging.DEBUG, 'submitjob: transfers=%s, job_params=%s' % ([str(t) for t in job['transfers']], job['job_params']))
-                        submit_transfer(transfertool_obj=transfertool_obj, transfers=job['transfers'], job_params=job['job_params'], submitter='transfer_submitter',
-                                        timeout=timeout, logger=logger)
-
-                if not once and total_transfers < group_bulk:
-                    logger(logging.DEBUG, 'Only %s transfers for %s which is less than group bulk %s, sleep %s seconds', total_transfers, activity, group_bulk, sleep_time)
-                    activity_next_exe_time[activity] = time.time() + sleep_time
-
-            except Exception:
-                logger(logging.CRITICAL, 'Exception', exc_info=True)
-                if once:
-                    raise
+    run_conveyor_daemon(
+        once=once,
+        graceful_stop=graceful_stop,
+        executable=executable,
+        logger_prefix=logger_prefix,
+        partition_wait_time=partition_wait_time,
+        sleep_time=sleep_time,
+        run_once_fnc=functools.partial(
+            run_once,
+            bulk=bulk,
+            group_bulk=group_bulk,
+            filter_transfertool=filter_transfertool,
+            transfertool=transfertool,
+            ignore_availability=ignore_availability,
+            scheme=scheme,
+            failover_scheme=failover_scheme,
+            partition_hash_var=partition_hash_var,
+            rse_ids=rse_ids,
+            timeout=timeout,
+            transfertool_kwargs=transfertool_kwargs,
+        ),
+        activities=activities,
+    )
 
 
 def stop(signum=None, frame=None):
