@@ -98,7 +98,8 @@ class TestBaseClient(unittest.TestCase):
         if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
             self.vo = {'vo': get_long_vo()}
             try:
-                remove(get_tmp_dir() + '/.rucio_root@%s/auth_token_root' % self.vo['vo'])
+                remove(get_tmp_dir()
+                       + '/.rucio_root@%s/auth_token_for_account_root' % self.vo['vo'])
             except OSError as error:
                 if error.args[0] != 2:
                     raise error
@@ -110,7 +111,7 @@ class TestBaseClient(unittest.TestCase):
         self.usercert = config_get('test', 'usercert')
         self.userkey = config_get('test', 'userkey')
         try:
-            remove(get_tmp_dir() + '/.rucio_root/auth_token_root')
+            remove(get_tmp_dir() + '/.rucio_root/auth_token_for_account_root')
         except OSError as error:
             if error.args[0] != 2:
                 raise error
