@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014-2021 CERN
+# Copyright 2014-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,8 @@
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
 # - Nick Smith <nick.smith@cern.ch>, 2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
 
 import itertools
 import unittest
@@ -107,7 +109,7 @@ class TestJudgeRepairer(unittest.TestCase):
 
         rule_repairer(once=True)  # Clean out the repairer
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse4_id, bytes=100)
+        files = create_files(3, scope, self.rse4_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -134,7 +136,7 @@ class TestJudgeRepairer(unittest.TestCase):
 
         rule_repairer(once=True)  # Clean out the repairer
         scope = InternalScope('mock', **self.vo)
-        files = create_files(4, scope, self.rse4_id, bytes=100)
+        files = create_files(4, scope, self.rse4_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -158,7 +160,7 @@ class TestJudgeRepairer(unittest.TestCase):
 
         rule_repairer(once=True)  # Clean out the repairer
         scope = InternalScope('mock', **self.vo)
-        files = create_files(4, scope, self.rse4_id, bytes=100)
+        files = create_files(4, scope, self.rse4_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -253,7 +255,7 @@ class TestJudgeRepairer(unittest.TestCase):
 
         rule_repairer(once=True)  # Clean out the repairer
         scope = InternalScope('mock', **self.vo)
-        files = create_files(4, scope, self.rse4_id, bytes=100)
+        files = create_files(4, scope, self.rse4_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -282,7 +284,7 @@ class TestJudgeRepairer(unittest.TestCase):
 
         rule_repairer(once=True)  # Clean out the repairer
         scope = InternalScope('mock', **self.vo)
-        files = create_files(4, scope, self.rse4_id, bytes=100)
+        files = create_files(4, scope, self.rse4_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -312,7 +314,7 @@ class TestJudgeRepairer(unittest.TestCase):
 
         region = make_region().configure(
             'dogpile.cache.memcached',
-            expiration_time=3600,
+            expiration_time=900,
             arguments={'url': config_get('cache', 'url', False, '127.0.0.1:11211'), 'distributed_lock': True}
         )
 
@@ -323,7 +325,7 @@ class TestJudgeRepairer(unittest.TestCase):
 
         for grouping, ignore_availability in itertools.product(["NONE", "DATASET", "ALL"], [True, False]):
             scope = InternalScope('mock', **self.vo)
-            files = create_files(1, scope, self.rse4_id, bytes=100)
+            files = create_files(1, scope, self.rse4_id, bytes_=100)
             dataset = 'dataset_' + str(uuid())
             add_did(scope, dataset, DIDType.DATASET, self.jdoe)
             attach_dids(scope, dataset, files, self.jdoe)
