@@ -50,9 +50,8 @@ from rucio.client.ruleclient import RuleClient
 from rucio.client.subscriptionclient import SubscriptionClient
 from rucio.common.config import config_get_bool
 from rucio.common.exception import (RuleNotFound, AccessDenied, InsufficientAccountLimit, DuplicateRule, RSEWriteBlocked,
-                                    RSEOverQuota,
-                                    RuleReplaceFailed, ManualRuleApprovalBlocked, InputValidationError,
-                                    UnsupportedOperation)
+                                    RSEOverQuota, RuleReplaceFailed, ManualRuleApprovalBlocked, InputValidationError,
+                                    UnsupportedOperation, InvalidValueForKey)
 from rucio.common.policy import get_policy
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid as uuid
@@ -78,14 +77,14 @@ from rucio.tests.common_server import get_vo
 LOG = getLogger(__name__)
 
 
-def create_files(nrfiles, scope, rse_id, bytes=1):
+def create_files(nrfiles, scope, rse_id, bytes_=1):
     """
     Creates a number of test files and add replicas to rse
 
     :param nrfiles:  Number of files to create
     :param scope:    Scope to create the files in
     :param rse_id:   RSE to add the replica to
-    :param bytes:    Bytes of each file
+    :param bytes_:    Bytes of each file
     :returns:        List of dict
     """
     if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
@@ -99,10 +98,10 @@ def create_files(nrfiles, scope, rse_id, bytes=1):
         file = 'file_%s' % uuid()
         if isinstance(rse_id, list):
             for r in rse_id:
-                add_replica(rse_id=r, scope=scope, name=file, bytes=bytes, account=jdoe)
+                add_replica(rse_id=r, scope=scope, name=file, bytes_=bytes_, account=jdoe)
         else:
-            add_replica(rse_id=rse_id, scope=scope, name=file, bytes=bytes, account=jdoe)
-        files.append({'scope': scope, 'name': file, 'bytes': bytes})
+            add_replica(rse_id=rse_id, scope=scope, name=file, bytes_=bytes_, account=jdoe)
+        files.append({'scope': scope, 'name': file, 'bytes': bytes_})
     return files
 
 
@@ -534,7 +533,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         account_counter_before = get_usage(self.rse1_id, self.jdoe)
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -551,7 +550,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         """ REPLICATION RULE (CORE): Test if the account counter is updated correctly when a rule is removed"""
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -573,7 +572,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         """ REPLICATION RULE (CORE): Test if the account counter is updated correctly when a rule is updated"""
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -600,7 +599,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         rse_counter_before = get_rse_counter(self.rse3_id)
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -617,40 +616,40 @@ class TestReplicationRuleCore(unittest.TestCase):
         """ REPLICATION RULE (CORE): Test if a rule fails correctly when local account limit conflict"""
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse3_id, bytes=100)
+        files = create_files(3, scope, self.rse3_id, bytes_=100)
         # local quota
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
 
-        set_local_account_limit(account=self.jdoe, rse_id=self.rse3_id, bytes=5)
+        set_local_account_limit(account=self.jdoe, rse_id=self.rse3_id, bytes_=5)
 
         pytest.raises(InsufficientAccountLimit, add_rule, dids=[{'scope': scope, 'name': dataset}], account=self.jdoe, copies=1, rse_expression=self.rse3, grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)
-        set_local_account_limit(account=self.jdoe, rse_id=self.rse3_id, bytes=-1)
+        set_local_account_limit(account=self.jdoe, rse_id=self.rse3_id, bytes_=-1)
 
     def test_rule_add_fails_account_global_limit(self):
         """ REPLICATION RULE (CORE): Test if a rule fails correctly when global account limit conflict"""
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse3_id, bytes=100)
+        files = create_files(3, scope, self.rse3_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
 
-        set_local_account_limit(account=self.jdoe, rse_id=self.rse3_id, bytes=400)
+        set_local_account_limit(account=self.jdoe, rse_id=self.rse3_id, bytes_=400)
         # check with two global limits - one breaking limit is enough to let the rule fail
-        set_global_account_limit(rse_expression='%s|MOCK2' % self.rse3, account=self.jdoe, bytes=400)
-        set_global_account_limit(rse_expression='%s|MOCK' % self.rse3, account=self.jdoe, bytes=10)
+        set_global_account_limit(rse_expression='%s|MOCK2' % self.rse3, account=self.jdoe, bytes_=400)
+        set_global_account_limit(rse_expression='%s|MOCK' % self.rse3, account=self.jdoe, bytes_=10)
         pytest.raises(InsufficientAccountLimit, add_rule, dids=[{'scope': scope, 'name': dataset}], account=self.jdoe, copies=1, rse_expression=self.rse3, grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)
 
-        set_local_account_limit(account=self.jdoe, rse_id=self.rse3_id, bytes=-1)
-        set_global_account_limit(rse_expression='%s|MOCK' % self.rse3, account=self.jdoe, bytes=-1)
-        set_global_account_limit(rse_expression='%s|MOCK2' % self.rse3, account=self.jdoe, bytes=-1)
+        set_local_account_limit(account=self.jdoe, rse_id=self.rse3_id, bytes_=-1)
+        set_global_account_limit(rse_expression='%s|MOCK' % self.rse3, account=self.jdoe, bytes_=-1)
+        set_global_account_limit(rse_expression='%s|MOCK2' % self.rse3, account=self.jdoe, bytes_=-1)
 
     def test_rule_add_fails_rse_limit(self):
         """ REPLICATION RULE (CORE): Test if a rule fails correctly when rse limit set"""
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -667,7 +666,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         """ REPLICATION RULE (CORE): Test dataset callback"""
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -687,7 +686,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         """ REPLICATION RULE (CORE): Test dataset callback should not be sent"""
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -706,7 +705,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         """ REPLICATION RULE (CORE): Test dataset callback with late close"""
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -726,7 +725,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         """ REPLICATION RULE (CORE): Test dataset callback with judge evaluator"""
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
 
@@ -750,7 +749,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         """ REPLICATION RULE (CORE): Test rule progress callback with judge evaluator"""
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(30, scope, self.rse1_id, bytes=100)
+        files = create_files(30, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
 
@@ -913,6 +912,31 @@ class TestReplicationRuleCore(unittest.TestCase):
 
         assert(get_rule(rule_id2)['state'] == RuleState.REPLICATING)
         assert(get_rule(rule_id)['child_rule_id'] == rule_id2)
+        assert(get_rule(rule_id2)['activity'] == get_rule(rule_id)['activity'])
+        assert(get_rule(rule_id2)['source_replica_expression'] == get_rule(rule_id)['source_replica_expression'])
+
+        pytest.raises(RuleReplaceFailed, move_rule, rule_id, self.rse4)
+
+    def test_move_rule_with_arguments(self):
+        """ REPLICATION RULE (CORE): Move a rule with activity and source-replica-expression specified"""
+        scope = InternalScope('mock', **self.vo)
+        files = create_files(3, scope, [self.rse1_id])
+        dataset = 'dataset_' + str(uuid())
+        add_did(scope, dataset, DIDType.DATASET, self.jdoe)
+        attach_dids(scope, dataset, files, self.jdoe)
+
+        rule_id = add_rule(dids=[{'scope': scope, 'name': dataset}], account=self.jdoe, copies=1, rse_expression=self.rse1, grouping='DATASET', weight=None, lifetime=None, locked=False, subscription_id=None)[0]
+
+        assert(get_rule(rule_id)['state'] == RuleState.OK)
+
+        activity = "No User Subscriptions"
+        source_replica_expression = self.rse3 + "|" + self.rse1
+        rule_id2 = move_rule(rule_id, self.rse3, activity=activity, source_replica_expression=source_replica_expression)
+
+        assert(get_rule(rule_id2)['state'] == RuleState.REPLICATING)
+        assert(get_rule(rule_id)['child_rule_id'] == rule_id2)
+        assert(get_rule(rule_id2)['activity'] == activity)
+        assert(get_rule(rule_id2)['source_replica_expression'] == source_replica_expression)
 
         pytest.raises(RuleReplaceFailed, move_rule, rule_id, self.rse4)
 
@@ -945,7 +969,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         rse_id = add_rse(rse, **self.vo)
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=200)
+        files = create_files(3, scope, self.rse1_id, bytes_=200)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -1025,7 +1049,7 @@ class TestReplicationRuleCore(unittest.TestCase):
     def test_release_rule(self):
         """ REPLICATION RULE (CORE): Test to release a parent rule after child rule is OK"""
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -1064,7 +1088,7 @@ class TestReplicationRuleCore(unittest.TestCase):
         scope = InternalScope('mock', **self.vo)
         archive = {'scope': scope, 'name': '%s.zip' % str(uuid()), 'type': 'FILE',
                    'bytes': 2596, 'adler32': 'beefdead'}
-        add_replica(rse_id=self.rse1_id, scope=scope, name=archive['name'], bytes=2596, account=self.jdoe)
+        add_replica(rse_id=self.rse1_id, scope=scope, name=archive['name'], bytes_=2596, account=self.jdoe)
         files_in_archive = [{'scope': scope, 'name': 'witrep-%i-%s' % (i, str(uuid())), 'type': 'FILE',
                              'bytes': 1234, 'adler32': 'deadbeef'} for i in range(2)]
         attach_dids(scope, archive['name'], files_in_archive, self.jdoe)
@@ -1077,11 +1101,11 @@ class TestReplicationRuleCore(unittest.TestCase):
         scope = InternalScope('mock', **self.vo)
         archive = {'scope': scope, 'name': '%s.zip' % str(uuid()), 'type': 'FILE',
                    'bytes': 2596, 'adler32': 'beefdead'}
-        add_replica(rse_id=self.rse1_id, scope=scope, name=archive['name'], bytes=2596, account=self.jdoe)
+        add_replica(rse_id=self.rse1_id, scope=scope, name=archive['name'], bytes_=2596, account=self.jdoe)
         files_in_archive = [{'scope': scope, 'name': 'witrep-%i-%s' % (i, str(uuid())), 'type': 'FILE',
                              'bytes': 1234, 'adler32': 'deadbeef'} for i in range(2)]
         attach_dids(scope, archive['name'], files_in_archive, self.jdoe)
-        add_replica(rse_id=self.rse1_id, scope=scope, name=files_in_archive[1]['name'], bytes=2596, account=self.jdoe)
+        add_replica(rse_id=self.rse1_id, scope=scope, name=files_in_archive[1]['name'], bytes_=2596, account=self.jdoe)
 
         add_rule(dids=[{'scope': scope, 'name': files_in_archive[1]['name']}], account=self.jdoe, copies=1, rse_expression=self.rse3, grouping='NONE',
                  weight=None, lifetime=None, locked=False, subscription_id=None)
@@ -1103,29 +1127,29 @@ class TestReplicationRuleCore(unittest.TestCase):
             #  6 replicas @ MOCK4 -> file11 .. file16
             #  5 replicas @ MOCK5 -> file17 .. file20, file25
             for i in range(1, 8):
-                add_replica(rse_id=get_rse_id('MOCK', **self.vo), scope=scope, name='file_%06d.data' % i, bytes=10000 + i, account=account)
+                add_replica(rse_id=get_rse_id('MOCK', **self.vo), scope=scope, name='file_%06d.data' % i, bytes_=10000 + i, account=account)
             for i in range(8, 11):
-                add_replica(rse_id=get_rse_id('MOCK3', **self.vo), scope=scope, name='file_%06d.data' % i, bytes=10000 + i, account=account)
+                add_replica(rse_id=get_rse_id('MOCK3', **self.vo), scope=scope, name='file_%06d.data' % i, bytes_=10000 + i, account=account)
             for i in range(11, 17):
-                add_replica(rse_id=get_rse_id('MOCK4', **self.vo), scope=scope, name='file_%06d.data' % i, bytes=10000 + i, account=account)
+                add_replica(rse_id=get_rse_id('MOCK4', **self.vo), scope=scope, name='file_%06d.data' % i, bytes_=10000 + i, account=account)
             for i in range(17, 21):
-                add_replica(rse_id=get_rse_id('MOCK5', **self.vo), scope=scope, name='file_%06d.data' % i, bytes=10000 + i, account=account)
+                add_replica(rse_id=get_rse_id('MOCK5', **self.vo), scope=scope, name='file_%06d.data' % i, bytes_=10000 + i, account=account)
             for i in range(21, 25):
-                add_replica(rse_id=get_rse_id('MOCK', **self.vo), scope=scope, name='file_%06d.data' % i, bytes=10000 + i, account=account)
+                add_replica(rse_id=get_rse_id('MOCK', **self.vo), scope=scope, name='file_%06d.data' % i, bytes_=10000 + i, account=account)
             for i in range(25, 26):
-                add_replica(rse_id=get_rse_id('MOCK5', **self.vo), scope=scope, name='file_%06d.data' % i, bytes=10000 + i, account=account)
+                add_replica(rse_id=get_rse_id('MOCK5', **self.vo), scope=scope, name='file_%06d.data' % i, bytes_=10000 + i, account=account)
 
-            add_did(scope=scope, name='ds1', type='DATASET', account=account)
+            add_did(scope=scope, name='ds1', did_type='DATASET', account=account)
             attach_dids(scope=scope, name='ds1', dids=[{'scope': scope, 'name': 'file_%06d.data' % i} for i in range(1, 10 + 1)], account=account)
-            add_did(scope=scope, name='ds2', type='DATASET', account=account)
+            add_did(scope=scope, name='ds2', did_type='DATASET', account=account)
             attach_dids(scope=scope, name='ds2', dids=[{'scope': scope, 'name': 'file_%06d.data' % i} for i in range(11, 20 + 1)], account=account)
-            add_did(scope=scope, name='ds3', type='DATASET', account=account)
+            add_did(scope=scope, name='ds3', did_type='DATASET', account=account)
             attach_dids(scope=scope, name='ds3', dids=[{'scope': scope, 'name': 'file_%06d.data' % i} for i in (list(range(21, 25 + 1)) + [1, 2, 11, 12])], account=account)
-            add_did(scope=scope, name='container12', type='CONTAINER', account=account)
+            add_did(scope=scope, name='container12', did_type='CONTAINER', account=account)
             attach_dids(scope=scope, name='container12', dids=[{'scope': scope, 'name': 'ds1'}, {'scope': scope, 'name': 'ds2'}, ], account=account)
-            add_did(scope=scope, name='container13', type='CONTAINER', account=account)
+            add_did(scope=scope, name='container13', did_type='CONTAINER', account=account)
             attach_dids(scope=scope, name='container13', dids=[{'scope': scope, 'name': 'ds1'}, {'scope': scope, 'name': 'ds3'}, ], account=account)
-            add_did(scope=scope, name='container1213', type='CONTAINER', account=account)
+            add_did(scope=scope, name='container1213', did_type='CONTAINER', account=account)
             attach_dids(scope=scope, name='container1213', dids=[{'scope': scope, 'name': 'container12'}, {'scope': scope, 'name': 'container13'}, ], account=account)
 
         account = self.jdoe
@@ -1400,3 +1424,125 @@ class TestReplicationRuleClient(unittest.TestCase):
         self.rule_client.approve_replication_rule(rule_id)
         rule = self.rule_client.get_replication_rule(rule_id)
         assert rule['state'] == RuleState.INJECT.name
+
+
+def test_add_rule_with_0_copies(did_client, did_factory, root_account, rse_factory):
+    """ REPLICATION RULE (CLIENT): Add a replication rule and list full history """
+    rse, rse_id = rse_factory.make_posix_rse()
+    file = did_factory.upload_test_file(rse)
+    dataset_internal = did_factory.make_dataset()
+    container_internal = did_factory.make_container()
+
+    # make all scopes external
+    file, dataset, container = ({'scope': did['scope'].external, 'name': did['name']} for did in (file, dataset_internal, container_internal))
+
+    # Attach dataset to container
+    did_client.add_files_to_dataset(files=[file], **dataset)
+    did_client.add_datasets_to_container(dsns=[dataset], **container)
+
+    with pytest.raises(InvalidValueForKey) as e:
+        add_rule(dids=[container_internal], account=root_account, copies=0, rse_expression='MOCK', grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)
+        assert "The number of copies for a replication rule should be greater than 0" in str(e)
+
+
+@pytest.mark.noparallel(reason='Asynchronos behavior when loading locks')
+def test_detach_dataset_lock_removal(did_client, did_factory, root_account, rse_factory, vo):
+    rse, rse_id = rse_factory.make_posix_rse()
+    file = did_factory.upload_test_file(rse)
+    dataset_internal = did_factory.make_dataset()
+    container_internal = did_factory.make_container()
+
+    # make all scopes external
+    file, dataset, container = ({'scope': did['scope'].external, 'name': did['name']} for did in (file, dataset_internal, container_internal))
+
+    # Attach dataset to container
+    did_client.add_files_to_dataset(files=[file], **dataset)
+    did_client.add_datasets_to_container(dsns=[dataset], **container)
+
+    add_rse_attribute(rse_id=rse_id, key='fakeweight', value=5)
+    add_rse_attribute(get_rse_id(rse='MOCK', vo=vo), "fakeweight", 5)
+
+    rule_id = add_rule(dids=[container_internal], account=root_account, copies=2, rse_expression='fakeweight>0', grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)[0]
+    print("Rule id: {0}".format(rule_id))
+    dataset_locks = list(get_dataset_locks(scope=dataset_internal['scope'], name=dataset['name']))
+    print("Dataset locks before detach: {0}".format(dataset_locks))
+    assert(len([d for d in dataset_locks if d["rule_id"] == rule_id]) == 2)
+
+    # Detach dataset from container, this should delete all locks on the dataset
+    did_client.detach_dids(**container, dids=[dataset_internal])
+
+    re_evaluator(once=True, did_limit=None)
+
+    dataset_locks = list(get_dataset_locks(**dataset_internal))
+    print("Dataset locks after detach: {0}".format(dataset_locks))
+    assert(len([d for d in dataset_locks if d["rule_id"] == rule_id]) == 0)
+
+
+@pytest.mark.noparallel(reason='Asynchronos behavior when loading locks')
+def test_detach_dataset_lock_removal_shared_dataset(did_client, did_factory, root_account, rse_factory, vo):
+    rse, rse_id = rse_factory.make_posix_rse()
+    file = did_factory.upload_test_file(rse)
+    dataset_internal = did_factory.make_dataset()
+    container_internal = did_factory.make_container()
+    container_internal_2 = did_factory.make_container()
+
+    # make all scopes external
+    file, dataset, container, container_2 = ({'scope': did['scope'].external, 'name': did['name']} for did in (file, dataset_internal, container_internal, container_internal_2))
+
+    # Attach dataset to container
+    did_client.add_files_to_dataset(files=[file], **dataset)
+    did_client.add_datasets_to_container(dsns=[dataset], **container)
+    did_client.add_datasets_to_container(dsns=[dataset], **container_2)
+
+    add_rse_attribute(rse_id=rse_id, key='fakeweight', value=5)
+    add_rse_attribute(get_rse_id(rse='MOCK', vo=vo), "fakeweight", 5)
+
+    rule_id = add_rule(dids=[container_internal], account=root_account, copies=2, rse_expression='fakeweight>0', grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)[0]
+    print("Rule id: {0}".format(rule_id))
+    dataset_locks = list(get_dataset_locks(scope=dataset_internal['scope'], name=dataset['name']))
+    print("Dataset locks before detach: {0}".format(dataset_locks))
+    assert(len([d for d in dataset_locks if d["rule_id"] == rule_id]) == 2)
+
+    # Detach dataset from container, this should delete all locks on the dataset
+    did_client.detach_dids(**container, dids=[dataset_internal])
+
+    re_evaluator(once=True, did_limit=None)
+
+    dataset_locks = list(get_dataset_locks(**dataset_internal))
+    print("Dataset locks after detach: {0}".format(dataset_locks))
+    assert(len([d for d in dataset_locks if d["rule_id"] == rule_id]) == 0)
+
+
+@pytest.mark.noparallel(reason='Asynchronos behavior when loading locks')
+def test_detach_dataset_lock_removal_shared_file(did_client, did_factory, root_account, rse_factory, vo):
+    rse, rse_id = rse_factory.make_posix_rse()
+    file = did_factory.upload_test_file(rse)
+    dataset_internal = did_factory.make_dataset()
+    dataset_internal_2 = did_factory.make_dataset()
+    container_internal = did_factory.make_container()
+
+    # make all scopes external
+    file, dataset, dataset_2, container = ({'scope': did['scope'].external, 'name': did['name']} for did in (file, dataset_internal, dataset_internal_2, container_internal))
+
+    # Attach dataset to container
+    did_client.add_files_to_dataset(files=[file], **dataset)
+    did_client.add_files_to_dataset(files=[file], **dataset_2)
+    did_client.add_datasets_to_container(dsns=[dataset], **container)
+
+    add_rse_attribute(rse_id=rse_id, key='fakeweight', value=5)
+    add_rse_attribute(get_rse_id(rse='MOCK', vo=vo), "fakeweight", 5)
+
+    rule_id = add_rule(dids=[container_internal], account=root_account, copies=2, rse_expression='fakeweight>0', grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)[0]
+    print("Rule id: {0}".format(rule_id))
+    dataset_locks = list(get_dataset_locks(scope=dataset_internal['scope'], name=dataset['name']))
+    print("Dataset locks before detach: {0}".format(dataset_locks))
+    assert(len([d for d in dataset_locks if d["rule_id"] == rule_id]) == 2)
+
+    # Detach dataset from container, this should delete all locks on the dataset
+    did_client.detach_dids(**container, dids=[dataset_internal])
+
+    re_evaluator(once=True, did_limit=None)
+
+    dataset_locks = list(get_dataset_locks(**dataset_internal))
+    print("Dataset locks after detach: {0}".format(dataset_locks))
+    assert(len([d for d in dataset_locks if d["rule_id"] == rule_id]) == 0)
